#!/bin/sh

# provide this script with a command. it will create a file called
# "trace" in your pwd which will contain ftrace information

# set FTRACE_NO_TRACE
# This will allow you to filter out events
# separate different events with colons and use *'s as wildcards
# This uses the set_ftrace_notrace option.
#
# set FTRACE_TRACE
# This will allow you to define events you want to have traced
# separate different events with colons and use *'s as wildcards
# This uses the set_ftrace_filter option.
#
# set FTRACE_BUFFER_SIZE
# This will define the per CPU buffer size of ftrace in kByte.
# The total buffer size will be 2 * $FTRACE_BUFFER_SIZE * <nr. of CPUs>
# This uses the buffer_size_kb option.
#



OPTIONS="print-parent
nosym-offset
nosym-addr
noverbose
noraw
nohex
nobin
noblock
nostacktrace
trace_printk
noftrace_preempt
nobranch
annotate
nouserstacktrace
nosym-userobj
noprintk-msg-only
context-info
nolatency-format
sleep-time
graph-time
record-cmd
overwrite
nodisable_on_free
funcgraph-overrun
funcgraph-cpu
funcgraph-overhead
funcgraph-proc
funcgraph-duration
funcgraph-abstime
funcgraph-irqs"



echo function_graph > /sys/kernel/debug/tracing/current_tracer
for OPTION in $OPTIONS
do
	echo $OPTION > /sys/kernel/debug/tracing/trace_options
done


echo 0 > /sys/kernel/debug/tracing/tracing_on

echo "" > /sys/kernel/debug/tracing/trace

if [ -n "$FTRACE_NO_TRACE" ]
then
	echo "applying filter, functions $FTRACE_NO_TRACE will not be traced"
	echo $FTRACE_NO_TRACE > /sys/kernel/debug/tracing/set_ftrace_notrace
fi


if [ -n "$FTRACE_TRACE" ]
then
	echo "applying filter, functions $FTRACE_TRACE will be traced"
	echo $FTRACE_TRACE > /sys/kernel/debug/tracing/set_ftrace_filter
fi

if [ -n "$FTRACE_BUFFER_SIZE" ]
then
	echo "setting buffer size to $FTRACE_BUFFER_SIZE"
	echo $FTRACE_BUFFER_SIZE > /sys/kernel/debug/tracing/buffer_total_size_kb
fi

TOTAL_BUFFER_SIZE=`cat /sys/kernel/debug/tracing/buffer_size_kb`
echo "starting trace, total buffer size=$TOTAL_BUFFER_SIZE"

# now it starts
echo 1 > /sys/kernel/debug/tracing/tracing_on

# execute command
$@

# stop
echo 0 > /sys/kernel/debug/tracing/tracing_on

cp /sys/kernel/debug/tracing/trace ./

echo nop > /sys/kernel/debug/tracing/current_tracer

